@@ -13,12 +13,26 @@ use super::{Number, _Complex};
 pub trait NumberClass: Default + Into<NumberType> + Ord + Send + fmt::Display {
     type Instance: NumberInstance;
 
+    /// Cast the given `Number` into an instance of this type.
     fn cast(&self, n: Number) -> Self::Instance;
 
+    /// Return `true` if this is a complex type.
+    fn is_complex(&self) -> bool {
+        return false;
+    }
+
+    /// Return `false` if this is a complex type.
+    fn is_real(&self) -> bool {
+        !self.is_complex()
+    }
+
+    /// Return the maximum size of this type of [`Number`], in bits.
     fn size(self) -> usize;
 
+    /// Return `1` as an instance of this type.
     fn one(&self) -> <Self as NumberClass>::Instance;
 
+    /// Return `0` as an instance of this type.
     fn zero(&self) -> <Self as NumberClass>::Instance;
 }
 
@@ -105,6 +119,34 @@ pub trait NumberInstance:
     }
 }
 
+pub trait Trigonometry {
+    type Out: NumberInstance;
+
+    fn asin(self) -> Self::Out;
+
+    fn sin(self) -> Self::Out;
+
+    fn asinh(self) -> Self::Out;
+
+    fn sinh(self) -> Self::Out;
+
+    fn acos(self) -> Self::Out;
+
+    fn cos(self) -> Self::Out;
+
+    fn acosh(self) -> Self::Out;
+
+    fn cosh(self) -> Self::Out;
+
+    fn atan(self) -> Self::Out;
+
+    fn tan(self) -> Self::Out;
+
+    fn atanh(self) -> Self::Out;
+
+    fn tanh(self) -> Self::Out;
+}
+
 /// Defines common operations on floating-point numeric types.
 pub trait FloatInstance {
     /// Return `true` if this `Number` is infinite (e.g. [`f32::INF`]).
@@ -138,6 +180,10 @@ impl NumberClass for ComplexType {
         }
     }
 
+    fn is_complex(&self) -> bool {
+        true
+    }
+
     fn size(self) -> usize {
         match self {
             Self::C32 => 8,
