@@ -35,7 +35,6 @@ mod class;
 mod instance;
 
 pub use class::*;
-use destream::Encoder;
 pub use instance::*;
 
 pub trait DType {
@@ -90,7 +89,8 @@ impl fmt::Display for Error {
 
 type _Complex<T> = num::complex::Complex<T>;
 
-const EXPECTING: &str = "a Number, like 1 or -2 or 3.14 or [0., -1.414]";
+const ERR_COMPLEX: &str = "a complex number";
+const ERR_NUMBER: &str = "a Number, like 1 or -2 or 3.14 or [0., -1.414]";
 
 /// A generic number.
 #[derive(Clone, Copy, Eq, Hash)]
@@ -446,6 +446,39 @@ impl Product for Number {
     }
 }
 
+macro_rules! trig {
+    ($fun:ident) => {
+        fn $fun(self) -> Self::Out {
+            match self {
+                Self::Bool(b) => b.$fun().into(),
+                Self::Complex(c) => c.$fun().into(),
+                Self::Float(f) => f.$fun().into(),
+                Self::Int(i) => i.$fun().into(),
+                Self::UInt(u) => u.$fun().into(),
+            }
+        }
+    };
+}
+
+impl Trigonometry for Number {
+    type Out = Self;
+
+    trig! {asin}
+    trig! {sin}
+    trig! {asinh}
+    trig! {sinh}
+
+    trig! {acos}
+    trig! {cos}
+    trig! {acosh}
+    trig! {cosh}
+
+    trig! {atan}
+    trig! {tan}
+    trig! {atanh}
+    trig! {tanh}
+}
+
 impl Default for Number {
     fn default() -> Self {
         Self::Bool(Boolean::default())
@@ -805,7 +838,7 @@ impl<'de> serde::de::Visitor<'de> for NumberVisitor {
     type Value = Number;
 
     fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.write_str(EXPECTING)
+        f.write_str(ERR_NUMBER)
     }
 
     #[inline]
@@ -872,11 +905,11 @@ impl<'de> serde::de::Visitor<'de> for NumberVisitor {
     fn visit_seq<A: serde::de::SeqAccess<'de>>(self, mut seq: A) -> Result<Self::Value, A::Error> {
         let re = seq
             .next_element()?
-            .ok_or_else(|| SerdeError::custom("Complex number missing real component"))?;
+            .ok_or_else(|| SerdeError::invalid_length(0, &ERR_COMPLEX))?;
 
         let im = seq
             .next_element()?
-            .ok_or_else(|| SerdeError::custom("Complex number missing imaginary component"))?;
+            .ok_or_else(|| SerdeError::invalid_length(1, &ERR_COMPLEX))?;
 
         Ok(Number::Complex(Complex::C64(_Complex::<f64>::new(re, im))))
     }
@@ -887,7 +920,7 @@ impl destream::de::Visitor for NumberVisitor {
     type Value = Number;
 
     fn expecting() -> &'static str {
-        EXPECTING
+        ERR_NUMBER
     }
 
     #[inline]
@@ -957,12 +990,12 @@ impl destream::de::Visitor for NumberVisitor {
         let re = seq
             .next_element(())
             .await?
-            .ok_or_else(|| DestreamError::custom("Complex number missing real component"))?;
+            .ok_or_else(|| DestreamError::invalid_length(0, ERR_COMPLEX))?;
 
         let im = seq
             .next_element(())
             .await?
-            .ok_or_else(|| DestreamError::custom("Complex number missing imaginary component"))?;
+            .ok_or_else(|| DestreamError::invalid_length(1, ERR_COMPLEX))?;
 
         Ok(Number::Complex(Complex::C64(_Complex::<f64>::new(re, im))))
     }
@@ -1001,7 +1034,7 @@ impl Serialize for Number {
 }
 
 impl<'en> ToStream<'en> for Number {
-    fn to_stream<E: Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
+    fn to_stream<E: destream::Encoder<'en>>(&'en self, e: E) -> Result<E::Ok, E::Error> {
         match self {
             Number::Bool(b) => b.to_stream(e),
             Number::Complex(c) => c.to_stream(e),
@@ -1013,7 +1046,7 @@ impl<'en> ToStream<'en> for Number {
 }
 
 impl<'en> IntoStream<'en> for Number {
-    fn into_stream<E: Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
+    fn into_stream<E: destream::Encoder<'en>>(self, e: E) -> Result<E::Ok, E::Error> {
         match self {
             Number::Bool(b) => b.into_stream(e),
             Number::Complex(c) => c.into_stream(e),
@@ -1177,6 +1210,7 @@ mod tests {
         assert_eq!(Number::from_str("12").unwrap(), Number::from(12));
         assert_eq!(Number::from_str("1e6").unwrap(), Number::from(1e6));
         assert_eq!(Number::from_str("1e-6").unwrap(), Number::from(1e-6));
+        assert_eq!(Number::from_str("1e-06").unwrap(), Number::from(1e-6));
         assert_eq!(Number::from_str("+31.4").unwrap(), Number::from(31.4));
         assert_eq!(Number::from_str("-3.14").unwrap(), Number::from(-3.14));
         assert_eq!(
